
local MAJOR = "StarLibError-1.0" 
local MINOR = 11
assert(LibStub, MAJOR.." requires LibStub") 
local StarLibError = LibStub:NewLibrary(MAJOR, MINOR)
if not StarLibError then return end
local pool = setmetatable({}, {__mode = "k"})
StarLibError.defaultTexts = {"Quiet",  "Normal", "Verbose"}

local DEFAULT_LEVEL = #StarLibError.defaultTexts 
-- 3 verbose
-- 2 warning
-- 1 quiet

if not StarLibError.__index then
	StarLibError.__index = StarLibError
	StarLibError.name = MAJOR
	StarLibError.errorLevel = DEFAULT_LEVEL
end

-- Create a new StarLibError object
-- @name StarLibError:New
-- @usage StarLibError:New(name, [handler])
-- @param name Give your error object a name
-- @param errorLevel Critical level for this object. This is optional. Default is 2.
-- @param handler An optional handler function, otherwise we use ChatFrame1 for printing
-- @return A new StarLibError object
function StarLibError:New(name, errorLevel, handler) 
	assert(name, "StarLibError.New requires a name")
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)
	
	obj.name = name
	obj.errorLevel = errorLevel or DEFAULT_LEVEL
	
	if type(handler) == "function" then
		obj.handler = handler	
	end
		
	return obj	
end

-- Delete a StarLibError object
-- @name StarLibError:Del
-- @usage object:Del() or StarLibError(object)
-- @return Nothing
function StarLibError:Del(err)
	if not err then err = self end
	pool[err] = true
end

-- Print something
-- @name StarLibError.Print
-- @usage LibError:Print(text, errorLevel)
-- @param text The text to print.
-- @param errorLevel The critical threshhold for this LibError object
-- @return Nothing
function StarLibError:Print(text, errorLevel)
	assert(self, "StarLibError failed printing due to invalid 'self'")
	assert(self.name and self.name ~= "", "StarLibError object has no name")
	assert(text, "StarLibError failed printing due to nil text (" .. self.name .. ")")
			
	if type(errorLevel) ~= "number" then
		errorLevel = DEFAULT_LEVEL
	end
	
	if self.handler then
		self.handler(text, errorLevel)
		return
	end
	
	local color
	if errorLevel == 3 then
		color = "|cffff0000"
	elseif errorLevel == 2 then
		color = "|cffffff00"
	elseif errorLevel == 1 then
		color = "|cff00ff00"
	else
		color = "|cffffffff"
	end
	
	if self.errorLevel >= errorLevel then
		ChatFrame1:AddMessage(("(%d)"):format(GetTime()) .. color .. (self.name or " ") .. "|r:: " .. text)		
	end
end